--
-- BogballeM2W_Base
-- Specialization for BogballeM2W_Base
--
-- @author:  	Manuel Leithner
-- @date:		25/10/10
-- @version:	v2.0
-- @history:	v1.0 - initial implementation
--				v2.0 - Converted to LS2011
--

BogballeM2W_Base = {};

function BogballeM2W_Base.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Fillable, specializations);
end;

function BogballeM2W_Base:load(xmlFile)

	self.changeSprayWidth = SpecializationUtil.callSpecializationsFunction("changeSprayWidth");
	self.changeCapacity = SpecializationUtil.callSpecializationsFunction("changeCapacity");
	self.changeLiterPerHa = SpecializationUtil.callSpecializationsFunction("changeLiterPerHa");

	self.attacherOptions = {};
	self.attacherOptions.pallet = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attacherJoint#pallet"));
	self.attacherOptions.frame = Utils.indexToObject(self.components, getXMLString(xmlFile, "vehicle.attacherJoint#frame"));
	self.attacherOptions.upY = getXMLFloat(xmlFile, "vehicle.attacherJoint#upY");
	self.attacherOptions.downY = getXMLFloat(xmlFile, "vehicle.attacherJoint#downY");
	self.attacherOptions.scale = 1;
	
	local settings = {};
	settings.literPerHa = {};
	settings.literPerHa.price = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.settings.literPerHa#price"), 0.1);
	settings.literPerHa.min = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.settings.literPerHa#min"), 50);
	settings.literPerHa.max = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.settings.literPerHa#max"), 300);
	settings.literPerHa.multiplier = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.settings.literPerHa#multiplier"), 6);
	settings.literPerHa.default = Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.settings.literPerHa#current"), 175);
	settings.literPerHa.current = settings.literPerHa.default;
	settings.minAreaWidth = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.settings.area#minAreaWidth"), 5);
	settings.maxAreaWidth = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.settings.area#maxAreaWidth"), 14);
	settings.step = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.settings.area#step"), 1);
	self.currentAreaWidth = settings.maxAreaWidth;
	self.settings = settings;
	
	self.spinners = {};
	local count = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.spinners#count"), 0);
    for i=1, count do
		local spinner = {};
        local name = string.format("vehicle.spinners.spinner" .. "%d", i);
	    spinner.node = Utils.indexToObject(self.components, getXMLString(xmlFile, name .. "#index"));
	    spinner.direction = Utils.getNoNil(getXMLInt(xmlFile, name .. "#direction"), 1);
		table.insert(self.spinners, spinner);
    end;

	local numworkAreasBow = Utils.getNoNil(getXMLInt(xmlFile, "vehicle.workAreasBow#count"), 0);
	self.workAreasBow = {};
    for i=1, numworkAreasBow do
        self.workAreasBow[i] = {};
        local areanamei = string.format("vehicle.workAreasBow.workArea%d", i);
        self.workAreasBow[i].start = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#startIndex"));
        self.workAreasBow[i].width = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#widthIndex"));
        self.workAreasBow[i].height = Utils.indexToObject(self.components, getXMLString(xmlFile, areanamei .. "#heightIndex"));
		self.workAreasBow[i].minStartX = getXMLFloat(xmlFile, areanamei .. "#minStartX");	
		self.workAreasBow[i].minHeightX = getXMLFloat(xmlFile, areanamei .. "#minHeightX");
		self.workAreasBow[i].minHeightZ = getXMLFloat(xmlFile, areanamei .. "#minHeightZ");
		local x,y,z = getTranslation(self.workAreasBow[i].start);
		self.workAreasBow[i].maxStartX = x;
		x,y,z = getTranslation(self.workAreasBow[i].height);
		self.workAreasBow[i].maxHeightX = x;
		self.workAreasBow[i].maxHeightZ = z;
    end;	
end;

function BogballeM2W_Base:delete()
end;

function BogballeM2W_Base:readStream(streamId, connection)
	local currentStep = streamReadInt8(streamId);
	local areaWidth = streamReadInt8(streamId);	 
	self.settings.literPerHa.current = streamReadInt16(streamId);
	self.settings.literPerHa.current = Utils.clamp(self.settings.literPerHa.current, self.settings.literPerHa.min, self.settings.literPerHa.max);
	
	local maxStep = self.currentStep;
	for i=currentStep, maxStep - 1 do
		self:changeCapacity(currentStep >= self.currentStep, true);
	end;
	local currentState = self.currentAreaWidth < areaWidth;
	while self.currentAreaWidth ~= areaWidth do 
		self:changeSprayWidth(self.currentAreaWidth < areaWidth, true);
		if self.currentAreaWidth >= self.settings.maxAreaWidth or self.currentAreaWidth <= self.settings.minAreaWidth or currentState ~= (self.currentAreaWidth < areaWidth) then
			break;
		end;
	end;
end;

function BogballeM2W_Base:writeStream(streamId, connection)
	streamWriteInt8(streamId, self.currentStep);
	streamWriteInt8(streamId, self.currentAreaWidth);
	streamWriteInt16(streamId, self.settings.literPerHa.current);	
end;

function BogballeM2W_Base:mouseEvent(posX, posY, isDown, isUp, button)
end;

function BogballeM2W_Base:keyEvent(unicode, sym, modifier, isDown)	
end;

function BogballeM2W_Base:update(dt)
	if self:getIsActive() then
		if self:getIsActiveForInput() then
			if InputBinding.hasEvent(InputBinding.bogballe_Increase_SprayWidth) then
				self:changeSprayWidth(true);
			end;
			if InputBinding.hasEvent(InputBinding.bogballe_Decrease_SprayWidth) then
				self:changeSprayWidth(false);
			end;
		end;

		if self.attachingFinished then
			local implement = self.attacherVehicle:getImplementByObject(self);
			if implement ~= nil then
				local joint = self.attacherVehicle.attacherJoints[implement.jointDescIndex];
				--self.attacherOptions.scale = joint.topArm.zScale;
				setJointFrame(joint.jointIndex, 0, self.attacherJoint.node);
				self.attachingFinished = false;			
			end;		
		end;
		
		if self.attacherVehicle ~= nil then
			--local lx,ly,lz = localDirectionToWorld(self.attacherVehicle.rootNode, 0, 0, -1*self.attacherOptions.scale);
			--local x, y, z = worldDirectionToLocal(getParent(self.attacherOptions.frame), lx,ly,lz);
			--setDirection(self.attacherOptions.frame, x, y, z, 0, 1, 0);		
		end;		
	end;
end;

function BogballeM2W_Base:updateTick(dt)
	if self:getIsActive() then	
		if self.isTurnedOn then 			
			local literPercentage = self.settings.literPerHa.current / self.settings.literPerHa.default;
			local widthPercentage = self.currentAreaWidth / self.settings.maxAreaWidth;
			local speedPercentage = math.max(1, ((self.lastSpeed * 3600) / 15));
			self.sprayLitersPerSecond[self.currentFillType] = self.settings.literPerHa.multiplier * literPercentage * widthPercentage * speedPercentage;
			
			for k,spinner in pairs(self.spinners) do
				rotate(spinner.node, 0, (-0.016 * spinner.direction)*dt, 0);
			end;
		end;
	end;	
end;

function BogballeM2W_Base:draw()
	local increase = g_i18n:getText("Key") .. " " .. InputBinding.getKeyNamesOfDigitalAction(InputBinding.bogballe_Increase_SprayWidth);
	local decrease = g_i18n:getText("Key") .. " " .. InputBinding.getKeyNamesOfDigitalAction(InputBinding.bogballe_Decrease_SprayWidth);
	g_currentMission:addExtraPrintText(string.format(g_i18n:getText("bogballe_workingWidth"), self.currentAreaWidth, decrease, increase));
end;

function BogballeM2W_Base:changeSprayWidth(increaseWidth, noEventSend) -- boolean	
	local stepWidth = self.settings.step;
	local changeDirection = 1;
	local oldWidth = self.currentAreaWidth;
	if not increaseWidth then
		changeDirection = -1;
	end;	
	self.currentAreaWidth = self.currentAreaWidth + (stepWidth * changeDirection);
	if self.currentAreaWidth >= self.settings.maxAreaWidth then
		self.currentAreaWidth = self.settings.maxAreaWidth;
	end;
	if self.currentAreaWidth <= self.settings.minAreaWidth then
		self.currentAreaWidth = self.settings.minAreaWidth;		
	end;
	
	if self.currentAreaWidth ~= oldWidth then
		BogballeChangeSprayWidthEvent.sendEvent(self, increaseWidth, noEventSend);
		
		local percentage = (self.currentAreaWidth - self.settings.minAreaWidth) / (self.settings.maxAreaWidth - self.settings.minAreaWidth);
		for k,workArea in pairs(self.workAreasBow) do 
			local minStartX = workArea.minStartX;
			if minStartX == nil then
				minStartX = self.workAreasBow[k-1].minHeightX;
			end;
		
			local x,y,z;
			local currentHeightX
			if workArea.minStartX ~= nil then
				x,y,z = getTranslation(workArea.start);
				currentHeightX = (workArea.maxStartX - workArea.minStartX) * percentage + workArea.minStartX;
				setTranslation(workArea.start,  currentHeightX, y, z);
			end;
		
			local area = workArea.start;
			if workArea.minStartX == nil then
				area = self.workAreasBow[k-1].start;
			end;			
			x,y,z = getTranslation(area);
			setTranslation(workArea.width,  0, 0, z);

			local currentHeightX = (workArea.maxHeightX - workArea.minHeightX) * percentage + workArea.minHeightX;
			local currentHeightZ = (workArea.maxHeightZ - workArea.minHeightZ) * percentage + workArea.minHeightZ;
			setTranslation(workArea.height,  currentHeightX, 0, currentHeightZ);
		end;
		
		local workArea = self.workAreas[1];
		setTranslation(workArea.start,   self.currentAreaWidth/2, 0, 0);
		setTranslation(workArea.width,  -self.currentAreaWidth/2, 0, 0);
		setTranslation(workArea.height,  self.currentAreaWidth/2, 0, -1);		
	end;
end;

function BogballeM2W_Base:changeLiterPerHa(increaseLiter, noEventSend) -- boolean
	local step = 10;
	local direction = 1;
	if not increaseLiter then
		direction = -1;
	end;
	
	local oldLiterPerHa = self.settings.literPerHa.current;
	self.settings.literPerHa.current = Utils.clamp(self.settings.literPerHa.current + (direction * step), self.settings.literPerHa.min, self.settings.literPerHa.max);
	if oldLiterPerHa ~= self.settings.literPerHa.current then
		BogballeChangeLiterPerHaEvent.sendEvent(self, increaseLiter, noEventSend);
	end;	
end;


function BogballeM2W_Base:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'currentStep="'..tostring(self.currentStep)..'" sprayWidth="' .. tostring(self.currentAreaWidth) .. '" literPerHa="' .. tostring(self.settings.literPerHa.current) ..'"';
	return attributes, nil;
end;